////////////////////////////////////////////////////////////////////////////
// Model KOSTKY.CPP               SIMLIB/C++
//
// Pklad simulanho programu - prodejna s Rubikovmi kostkami (verze 1)
//
// Do hrakstv pivezli Rubikovy kostky: rno v 8 hodin 500 kus
// a odpoledne  v 15  hodin opt  500 kus.  V hrakstv se otevr
// v 8 hodin rno a zavr se  v 18 hodin veer. Zkaznci pichzeli
// od  8  hodin  prmrn  po  3/4 minuty (s  exponencilnm
// rozloenm) a  kad zkaznk  koupil jen jednu kostku. Kdy
// u byly  vechny kostky  vyprodny, zkaznci  se  nevzdali, stli
// zpsobn ve front  a ekali, a  pivezou dalch  500 kus odpo-
// ledne v 15 hodin. Mezitm  ovem pichzeli nov zkaznci. Urete
// max. dlku fronty  a prmrnou  dobu ekn, spotejte, kolik z-
// kaznk  odelo  s nepozenou, protoe vedouc prodejnu  striktn
// zavela  v 18 hodin  a nebrala  ohled  na lidi, kte  u stli ve
// front. Po 18.hodin u dal zkaznci nepichzeli. Doba obsluhy
// je 1 minuta.
//

#include "simlib.h"

// doba simulace :  T_POC - poten, T_KON - konen
#define T_POC  8.0  // hodin
#define T_KON 18.0  // hodin

// deklarace konstant
#define T_OBSLUHY  1.0/60    // doba obsluhy zkaznka - 1 minuta
#define T_DODAVKY1 T_POC     // as dodvky kostek - rno
#define T_DODAVKY2 15.0      // as dodvky kostek - odpoledne
#define T_PRICH    0.75/60   // prmrn interval pchodu zkaznk 3/4 min

// deklarace globalnich objektu
Facility  Prodejna;           // prodejna kostek
Histogram Tabulka ("ekac doby zkaznk",0,0.15,20);
                              // histogram pro zznam ekacch dob zkaznk
unsigned Kostek = 0;          // poet kostek v prodejn
unsigned PocZakazniku = 0;    // celkov poet neobslouench zkaznk


class Zakaznik : public Process {  // tda zkaznk
  double Prichod;                  // as pchodu kadho zkaznka
  void Behavior() {                // popis chovn zkaznka
    Prichod = Time;
    PocZakazniku ++;
    Seize(Prodejna);
    WaitUntil (Kostek > 0);        // ekej, jestlie kostky nejsou
    Kostek --;
    Wait(T_OBSLUHY);
    Release(Prodejna);
    PocZakazniku --;
    Tabulka(Time - Prichod);       // zznam doby ekn do tabulky
  }
};

class Generator : public Event {        // genertor zkaznk
  void Behavior() {                     // popis chovn genertoru
    (new Zakaznik)->Activate();         // nov zkaznk
    Activate(Time+Exponential(T_PRICH));
  }
};

class Dodavka : public Event {          // dodvka kostek
  void Behavior() {
     Kostek = Kostek + 500;
     Terminate();
  }
};


int main() {
  SetOutput("kostky.out");             // nastaven vstupnho souboru
  Prodejna.SetName("Prodejna");
  Init(T_POC,T_KON);                   // inicializace experimentu
  (new Dodavka)->Activate(T_DODAVKY1); // inicializace dodvky zbo rno
  (new Dodavka)->Activate(T_DODAVKY2); // inicializace dodvky zbo odpoledne
  (new Generator)->Activate();         // inicializace genertoru zkaznk
  _Print(" KOSTKY - simulace obchodu\n");
  Run();                               // simulace
  Print("\n S nepozenou odelo %u zkaznk \n\n", PocZakazniku);
  Prodejna.Output();                   // tisk vsledk
  Tabulka.Output();
  return 0;
}

// konec
